/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer*
* Copyright (C) 2013-2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name : iodefine.h
* $Rev: $
* $Date::                           $
* Description : Definition of I/O Register (V1.00a)
******************************************************************************/
#ifndef R7S72100_IODEFINE_H
#define R7S72100_IODEFINE_H

#define IODEFINE_H_VERSION  100

enum iodefine_byte_select_t
{
    L = 0, H = 1,
    LL= 0, LH = 1, HL = 2, HH = 3
};

/***********************************************************************
  <<< [iodefine_reg32_t] >>> 
- Padding : sizeof(iodefine_reg32_t) == 4
- Alignment(Offset) : &UINT32==0, &UINT16[0]==0, &UINT16[1]==2
-                     &UINT8[0]==0, &UINT8[1]==1, &UINT8[2]==2, &UINT8[3]==3
- Endian : Independent (Same as CPU endian as register endian)
- Bit-Order : Independent
************************************************************************/
/* ->MISRA 18.4 : Pack unpack union */ /* ->SEC M1.6.2 */
/* ->SEC M1.10.1 : Not magic number */
union iodefine_reg32_t
{
    volatile uint32_t  UINT32;                                  /*  32-bit Access   */
    volatile uint16_t  UINT16[2];                               /*  16-bit Access   */
    volatile uint8_t   UINT8[4];                                /*  8-bit Access    */
};
/* <-SEC M1.10.1 */
/* <-MISRA 18.4 */ /* <-SEC M1.6.2 */

/***********************************************************************
  <<< [iodefine_reg32_16_t] >>> 
- Padding : sizeof(iodefine_reg32_16_t) == 4
- Alignment(Offset) : &UINT32==0, &UINT16[0]==0, &UINT16[1]==2
- Endian : Independent (Same as CPU endian as register endian)
- Bit-Order : Independent
************************************************************************/
/* ->MISRA 18.4 : Pack unpack union */ /* ->SEC M1.6.2 */
/* ->SEC M1.10.1 : Not magic number */
union iodefine_reg32_16_t
{
    volatile uint32_t  UINT32;                                  /*  32-bit Access   */
    volatile uint16_t  UINT16[2];                               /*  16-bit Access   */
};
/* <-SEC M1.10.1 */
/* <-MISRA 18.4 */ /* <-SEC M1.6.2 */

/***********************************************************************
  <<< [iodefine_reg16_8_t] >>> 
- Padding : sizeof(iodefine_reg16_8_t) == 2
- Alignment(Offset) : &UINT16==0, &UINT8[0]==0, &UINT8[1]==1
- Endian : Independent (Same as CPU endian as register endian)
- Bit-Order : Independent
************************************************************************/
/* ->MISRA 18.4 : Pack unpack union */ /* ->SEC M1.6.2 */
/* ->SEC M1.10.1 : Not magic number */
union iodefine_reg16_8_t
{
    volatile uint16_t  UINT16;                                  /*  16-bit Access   */
    volatile uint8_t   UINT8[2];                                /*  8-bit Access    */
};
/* <-SEC M1.10.1 */
/* <-MISRA 18.4 */ /* <-SEC M1.6.2 */






#include <mbed/TARGET_RZ_A1H/adc_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/bsc_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/ceu_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/cpg_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/disc_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/dmac_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/dvdec_iodefine.h>                           /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/ether_iodefine.h>                           /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/flctl_iodefine.h>                           /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/gpio_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/ieb_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/inb_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/intc_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/irda_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/jcu_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/l2c_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/lin_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/lvds_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/mlb_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/mmc_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/mtu2_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/ostm_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/pfv_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/pwm_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/riic_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/romdec_iodefine.h>                          /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/rscan0_iodefine.h>                          /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/rspi_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/rtc_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/scif_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/scim_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/scux_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/sdg_iodefine.h>                             /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/spdif_iodefine.h>                           /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/spibsc_iodefine.h>                          /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/ssif_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/usb20_iodefine.h>                           /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/vdc5_iodefine.h>                            /* (V1.00a) */
#include <mbed/TARGET_RZ_A1H/wdt_iodefine.h>                             /* (V1.00a) */
#endif
