/*
 * C
 *
 * Copyright 2015-2017 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "LLCOMM_UART.h"
#define   LLCOMM_BUFFERED_CONNECTION_IMPL LLCOMM_UART
#include "LLCOMM_BUFFERED_CONNECTION_impl.h"
#include "interrupts.h"

#include "RZA1H_serial.h"
#include "scif_iodefine.h"

/* Interrupt callback --------------------------------------------------------*/

void LLCOMM_UART_callback(uint32_t id, SerialIrq event)
{
	LLCOMM_UART* llcomm = (LLCOMM_UART*)id;

	// retrieve LLCOM environment for given comIndex
	LLCOMM_BUFFERED_CONNECTION* env = &llcomm->header;
	serial_t* handle = &llcomm->hal_uart_handle;

	uint8_t leave = interrupt_enter();

	// check RX
	if (event == RxIrq)
	{
		// read data, clear the interrupt and send data to MicroEJ framework
		LLCOMM_BUFFERED_CONNECTION_dataReceived(env, (int32_t)serial_getc(handle));
	}

	// check TX
	// warning: LLCOMM_BUFFERED_CONNECTION_transmitBufferReady() will disable the
	// TX interrupt but the a pending TX interrupt can occur after the disable
	// action -> use a boolean to ensure to not re-enter in function
	// LLCOMM_BUFFERED_CONNECTION_transmitBufferReady()
	if (llcomm->txInterruptEnabled && event == TxIrq)
	{
		// transmit buffer empty -> notify MicroEJ framework (it will disable the TX interrupt)
		LLCOMM_BUFFERED_CONNECTION_transmitBufferReady(env);
	}

	interrupt_leave(leave);
}

/* API -----------------------------------------------------------------------*/

void LLCOMM_BUFFERED_CONNECTION_IMPL_initialize(LLCOMM_BUFFERED_CONNECTION* env)
{
	// pinout, clocks and interrupts have been already initialized 
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getPlatformId(LLCOMM_BUFFERED_CONNECTION* env)
{
	return ((LLCOMM_UART*)env)->platformId;
}

void* LLCOMM_BUFFERED_CONNECTION_IMPL_getName(LLCOMM_BUFFERED_CONNECTION* env)
{
	// this kind of connection cannot be open from a name (only from a platform port number)
	return 0;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_configureDevice(LLCOMM_BUFFERED_CONNECTION* env, int32_t baudrate, int32_t bitsperchar, int32_t stopbits, int32_t parity)
{
	serial_t* handle = &(((LLCOMM_UART*)env)->hal_uart_handle);
	SerialParity selectedParity = ParityNone;
	int selectedDataBytes = 8;
	int selectedBitStop = 1;

	// get number of bits
	switch (bitsperchar)
	{
	default:
	case LLCOMM_BUFFERED_CONNECTION_DATABITS_5:
	case LLCOMM_BUFFERED_CONNECTION_DATABITS_6:
		// not available
		return LLCOMM_BUFFERED_CONNECTION_ERROR_INIT_UNSUPPORTED_BITSPERCHAR;

	case LLCOMM_BUFFERED_CONNECTION_DATABITS_7:
		selectedDataBytes = 7;
		break;

	case LLCOMM_BUFFERED_CONNECTION_DATABITS_8:
		selectedDataBytes = 8;
		break;

	case LLCOMM_BUFFERED_CONNECTION_DATABITS_9:
		// Not available
		return LLCOMM_BUFFERED_CONNECTION_ERROR_INIT_UNSUPPORTED_BITSPERCHAR;
		break;
	}

	// get number of stop bits
	switch (stopbits)
	{
	case LLCOMM_BUFFERED_CONNECTION_STOPBITS_1:
		selectedBitStop = 1 ;
		break ;

	case LLCOMM_BUFFERED_CONNECTION_STOPBITS_2:
		selectedBitStop = 2;
		break ;

	default:
		return LLCOMM_BUFFERED_CONNECTION_ERROR_INIT_UNSUPPORTED_STOPBITS;
	}

	// get parity
	switch (parity)
	{
	case LLCOMM_BUFFERED_CONNECTION_PARITY_EVEN:
		selectedParity = ParityEven;
		break ;

	case LLCOMM_BUFFERED_CONNECTION_PARITY_NO:
		selectedParity = ParityNone;
		break ;

	case LLCOMM_BUFFERED_CONNECTION_PARITY_ODD:
		selectedParity = ParityOdd;
		break ;

	default:
		return LLCOMM_BUFFERED_CONNECTION_ERROR_INIT_UNSUPPORTED_PARITY;
	}

	/* Set the configuration*/
	serial_baud(handle, baudrate);
	serial_format(handle, selectedDataBytes, selectedParity, selectedBitStop);

	return LLCOMM_BUFFERED_CONNECTION_SUCCESS;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableRX(LLCOMM_BUFFERED_CONNECTION* env)
{
	((LLCOMM_UART*)env)->hal_uart_handle.serial.uart->SCSCR |= RZAH1_SERIAL_UART_SCSCR_RE;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableRX(LLCOMM_BUFFERED_CONNECTION* env)
{
	((LLCOMM_UART*)env)->hal_uart_handle.serial.uart->SCSCR &= ~RZAH1_SERIAL_UART_SCSCR_RE;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableRXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	serial_irq_set(&(((LLCOMM_UART*)env)->hal_uart_handle), RxIrq, 1);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableRXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	serial_irq_set(&(((LLCOMM_UART*)env)->hal_uart_handle), RxIrq, 0);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableTX(LLCOMM_BUFFERED_CONNECTION* env)
{
	((LLCOMM_UART*)env)->hal_uart_handle.serial.uart->SCSCR |= RZAH1_SERIAL_UART_SCSCR_TE;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableTX(LLCOMM_BUFFERED_CONNECTION* env)
{
	((LLCOMM_UART*)env)->hal_uart_handle.serial.uart->SCSCR &= ~RZAH1_SERIAL_UART_SCSCR_TE;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_enableTXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	((LLCOMM_UART*)env)->txInterruptEnabled = 1;
	((LLCOMM_UART*)env)->hal_uart_handle.serial.uart->SCSCR |= RZAH1_SERIAL_UART_SCSCR_TIE;  // Set TIE
	serial_irq_set(&(((LLCOMM_UART*)env)->hal_uart_handle), TxIrq, 1);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_disableTXDeviceInterrupt(LLCOMM_BUFFERED_CONNECTION* env)
{
	serial_irq_set(&(((LLCOMM_UART*)env)->hal_uart_handle), TxIrq, 0);
	((LLCOMM_UART*)env)->txInterruptEnabled = 0;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getRXBufferStartAddress(LLCOMM_BUFFERED_CONNECTION* env)
{
	return (int32_t)(((LLCOMM_UART*)env)->rxBuffer);
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getRXBufferLength(LLCOMM_BUFFERED_CONNECTION* env)
{
	return LLCOMM_UART_RX_BUFFER_SIZE;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getTXBufferStartAddress(LLCOMM_BUFFERED_CONNECTION* env)
{
	return (int32_t)(((LLCOMM_UART*)env)->txBuffer);
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getTXBufferLength(LLCOMM_BUFFERED_CONNECTION* env)
{
	return LLCOMM_UART_TX_BUFFER_SIZE;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_useTXHardwareFIFO(LLCOMM_BUFFERED_CONNECTION* env)
{
	return LLCOMM_BUFFERED_CONNECTION_NO_HARDWARE_FIFO;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_txFifoFull(LLCOMM_BUFFERED_CONNECTION* env)
{
	return LLCOMM_BUFFERED_CONNECTION_HARDWARE_FIFO_NOT_FULL;
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_sendData(LLCOMM_BUFFERED_CONNECTION* env, int32_t data)
{
	/* Transmit Data */
	serial_putc(&(((LLCOMM_UART*)env)->hal_uart_handle), (data & (uint16_t)0x01FF));
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_isTransmissionComplete(LLCOMM_BUFFERED_CONNECTION* env)
{
	return (IS_REG_BIT_SET(((LLCOMM_UART*)env)->hal_uart_handle.serial.uart->SCFSR, RZAH1_SERIAL_UART_SCFSR_TEND))? LLCOMM_BUFFERED_CONNECTION_TRANSMISSION_COMPLETE : LLCOMM_BUFFERED_CONNECTION_TRANSMISSION_TIMEOUT;
}

int32_t LLCOMM_BUFFERED_CONNECTION_IMPL_getNbProperties(LLCOMM_BUFFERED_CONNECTION* env)
{
	// This implementation does not provide Device properties
	return 0;
}

void* LLCOMM_BUFFERED_CONNECTION_IMPL_getProperty(LLCOMM_BUFFERED_CONNECTION* env, void* propertyName)
{
	// This implementation does not provide Device properties
	return 0;
}

void* LLCOMM_BUFFERED_CONNECTION_IMPL_getPropertyName(LLCOMM_BUFFERED_CONNECTION* env, int32_t propertyId)
{
	// This implementation does not provide Device properties
	while(1);
}

void LLCOMM_BUFFERED_CONNECTION_IMPL_released(LLCOMM_BUFFERED_CONNECTION* env)
{
	// This implementation does not support dynamic connections add/removal.
	// To enable dynamic connections, synchronization functions must be implemented.
	// See LLCOMM_impl.h
	while(1);
}

