/*
 * C
 *
 * Copyright 2015-2017 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include <stdint.h>
#include "LLCOMM_UART.h"
#include "LLCOMM_impl.h"
#include "microej.h"
#include "cmsis_os.h"
#include "interrupts.h"

/* Methods declaration -------------------------------------------------------*/
void LLCOMM_UART_new(LLCOMM_BUFFERED_CONNECTION* env);

/* Enumerations --------------------------------------------------------------*/

/*
 * Defines the mapping between the HAL UART instance and MicroEJ LLCOMM_UARTs
 * table. The indexes are 0-based.
 */
typedef enum
{
	UART0_INDEX = 0,
} hal_to_microej_uart_table;

// UART0 Pin names
PinName UART0_TX  = P2_14;
PinName UART0_RX  = P2_15;

/* Fields --------------------------------------------------------------------*/

/*
 * Array of all static LLCOMM UARTs.
 */
static LLCOMM_UART LLCOMM_UARTs[1];

/* Private functions ---------------------------------------------------------*/

/*
 * Initialize LLCOMM_UARTs array for a specific HAL UART instance and initialize
 * corresponding MicroEJ LLCOMM_UART.
 */
static void COMMS_initialize_llcomm_uart(int32_t index, PinName tx, PinName rx, uint8_t platform_id)
{
	// store HAL UART instance

	serial_init(&(LLCOMM_UARTs[index].hal_uart_handle), tx, rx);

	// store HAL UART ID
	LLCOMM_UARTs[index].platformId = platform_id;

	// create MicroEJ UART connection
	LLCOMM_UART_new(&LLCOMM_UARTs[index].header);

	// add MicroEJ UART connection into MicroEJ framework as generic connection
	LLCOMM_addConnection((LLCOMM_CONNECTION*)&LLCOMM_UARTs[index].header);
}

static void COMMS_initialize_interrupt(LLCOMM_UART* llcomm)
{
	serial_irq_handler(&(llcomm->hal_uart_handle), LLCOMM_UART_callback, (uint32_t)llcomm);
	serial_irq_set(&(llcomm->hal_uart_handle), RxIrq, 1);
	serial_irq_set(&(llcomm->hal_uart_handle), TxIrq, 1);
}

/* Interrupt functions -------------------------------------------------------*/


/* Public functions ----------------------------------------------------------*/

/*
 * The semaphore must be created before the first call of LLCOMM_IMPL_syncConnectionsEnter()
 * which is called BEFORE LLCOMM_IMPL_initialize().
 */
void LLCOMM_stack_initialize(void)
{
	/* Not needed for static serial port */
}

void LLCOMM_IMPL_initialize(void)
{	
	// GR-PEACH port COM1: use UART0
	// 	- TX: P2_14
	// 	- RX: P2_15
	COMMS_initialize_llcomm_uart(UART0_INDEX, UART0_TX, UART0_RX, 1);
	COMMS_initialize_interrupt(&LLCOMM_UARTs[UART0_INDEX]);

}

void LLCOMM_IMPL_syncConnectionsEnter(void)
{
	/* Not needed for static serial port */
}

void LLCOMM_IMPL_syncConnectionsExit(void)
{
	/* Not needed for static serial port */
}
