/*
 * Copyright 2017 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#include <stdint.h>
#include <stdio.h>
#include "writeUIDFlash.h"
#include "generateUID.h"
#include "rt_TypeDef.h"   /* Support typdef in rt_System.h */
#include "rt_System.h"
#include "cmsis_os.h"
#include "mbed_interface.h"
#include "device_uid.h"

/* private  API ------------------------------------------------------------ */

static uint8_t isMemoryUIDValid(uint8_t * pFlashUIDAddr)
{
	uint8_t ret=0;
	uint16_t checksum = 0;
	for (int i=0; i<(GRPEACH_FLASH_UID_ADDR_SIZE); i++)
	{
		checksum += pFlashUIDAddr[i];
	}
	if (checksum != (((uint16_t)pFlashUIDAddr[GRPEACH_FLASH_UID_ADDR_CHECKSUM_MSB]<<8)+(uint16_t)pFlashUIDAddr[GRPEACH_FLASH_UID_ADDR_CHECKSUM_LSB]))
	{
		ret = 0;
		printf("ERROR : isMemoryUIDValid : UID checksum invalid\n");
	}
	else
	{
		ret = 1;
	}

	return ret;
}

/* API ------------------------------------------------------------ */

void init_device_uid(void)
{
	uint8_t * flashUIDAddr = (uint8_t *)(GRPEACH_FLASH_UID_ADDR_START);
	if (!isMemoryUIDValid(flashUIDAddr))
	{
		//Generate UID using noise on ADC pins
		uint8_t uid[GRPEACH_FLASH_UID_ADDR_SIZE+GRPEACH_FLASH_UID_CHECKSUM_SIZE];
		generateUID(uid, sizeof(uid));

		// Compute checksum that replace the last 2 bytes
		uint16_t sum = 0;
		for (uint8_t i =0; i<(GRPEACH_FLASH_UID_ADDR_SIZE); i++){
			sum += uid[i];
		}
		uid[GRPEACH_FLASH_UID_ADDR_CHECKSUM_MSB] = ((sum >> 8) & 0xFF);
		uid[GRPEACH_FLASH_UID_ADDR_CHECKSUM_LSB] = (sum & 0xFF);

		//Suspend system and stop interrupt
		rt_tsk_lock();
		int irq_dis = __disable_irq();\

		//write UID to Flash
		int32_t writeStatus = write_uid_to_flash(&(uid[0]));

		//Restart system
        if(!irq_dis) __enable_irq();
		rt_tsk_unlock();

		if (writeStatus == 0){
			printf("New UID written to Flash memory\n");
		}else{
			printf("ERROR : Failed to write in flash memory : %d", writeStatus);
		}
	}
}
