/*
 * Copyright 2017 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#include <mbed/mbed.h>
#include "cmsis_os.h"

#ifdef __cplusplus
	extern "C" {
#endif

#include "sha1.h"

#ifdef __cplusplus
	}
#endif

static AnalogIn pinA0(A0);
static AnalogIn pinA1(A1);
static AnalogIn pinA2(A2);
static AnalogIn pinA3(A3);

/* Private functions  --------------------------------------------------------*/
static uint16_t readPinA0(void){
	return pinA0.read_u16();
}

static uint16_t readPinA1(void){
	return pinA1.read_u16();
}

static uint16_t readPinA2(void){
	return pinA2.read_u16();
}

static uint16_t readPinA3(void){
	return pinA3.read_u16();
}

#ifdef __cplusplus
	extern "C" {
#endif

uint32_t generateUID(uint8_t *buffer, uint32_t len) {
	SHA1Context sha1;

	if (buffer == NULL) {
		return 0;
	}
	if (len > 20) {
		// cannot generate more than the 20 bytes of a SHA1
		return 0;
	}

	// SHA1 needs some initialization
	SHA1Reset(&sha1);

	// Updates with 4 analog values read on 4 different pins
	// right-shifted by 4 bits because of the way the value is computed
	// by the MBED APIs (see the impl of read_u16)
	for (int i = 0; i<1024; i++) {
		uint8_t readADC[4];
		readADC[0] = ((readPinA0() >> 4) & 0x00FF);
		readADC[1] = ((readPinA1() >> 4) & 0x00FF);
		readADC[2] = ((readPinA2() >> 4) & 0x00FF);
		readADC[3] = ((readPinA3() >> 4) & 0x00FF);
		SHA1Input(&sha1, readADC, 4);
	}

	// get as much as requested bytes into the destination buffer
	if (SHA1Result(&sha1)) {
		memcpy(buffer, sha1.Message_Digest, len);
	} else {
		// something went wrong, the SHA1 was corrupted
		return 0;
	}
}

#ifdef __cplusplus
	}
#endif
