/*
 * hal_peach.cpp
 *
 * Copyright 2017 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/
#include "mbed.h"

#ifdef __cplusplus
	extern "C" {
#endif

#include "LLHAL_GPIO_impl.h"

#ifdef __cplusplus
	}
#endif



/* Defines -------------------------------------------------------------------*/

/* Enum ----------------------------------------------------------------------*/

typedef enum
{

	/*
	 * List of MCU ports used by this implementation
	 */
	LLHAL_GPIO_PORT_1 = 1,
	LLHAL_GPIO_PORT_2 = 2,
	LLHAL_GPIO_PORT_3 = 3,
	LLHAL_GPIO_PORT_4 = 4,
	LLHAL_GPIO_PORT_5 = 5,
	LLHAL_GPIO_PORT_7 = 7,
	LLHAL_GPIO_PORT_8 = 8,
	LLHAL_GPIO_PORT_10 = 10,
	LLHAL_GPIO_PORT_11 = 11,

	/*
	* List of standard ports used by this implementation
	*/
	LLHAL_GPIO_PORT_ARDUINO_DIGITAL = 30,
	LLHAL_GPIO_PORT_ARDUINO_ANALOG = 31,

	/*
	* List of specific ports used by this implementation
	*/
		LLHAL_GPIO_PORT_CN9 = 69,
		LLHAL_GPIO_PORT_CN14 = 74,
		LLHAL_GPIO_PORT_CN15 = 75,

		LLHAL_GPIO_PORT_CN8 = 68,
		LLHAL_GPIO_PORT_CN11 = 71,
		LLHAL_GPIO_PORT_CN12 = 72,
		LLHAL_GPIO_PORT_CN13 = 73,
		LLHAL_GPIO_PORT_CN16 = 76,
		LLHAL_GPIO_PORT_CN17 = 77,

}HAL_GPIO_IMPL_Port;

/* Public functions ----------------------------------------------------------*/
int32_t hal_get_gpio_data(int32_t port, int32_t pin, gpio_t* gpio)
{
	switch (port)
	{
	case LLHAL_GPIO_PORT_1:
	{
		/*
		 * GPIOA check and initialization
		 */
		switch (pin)
		{
		case 0:
			gpio->pin = P1_0;
			break;
		case 1:
			gpio->pin = P1_1;
			break;
		case 2:
			gpio->pin = P1_2;
			break;
		case 3:
			gpio->pin = P1_3;
			break;
		case 6:
			gpio->pin = P1_6;
			break;
		case 7:
			gpio->pin = P1_7;
			break;
		case 8:
			gpio->pin = P1_8; //AN0
			break;
		case 9:
			gpio->pin = P1_9; //AN1
			break;
		case 10:
			gpio->pin = P1_10; //AN2
			break;
		case 11:
			gpio->pin = P1_11; //AN3
			break;
		case 12:
			gpio->pin = P1_12;
			break;
		case 13:
			gpio->pin = P1_13; //AN4
			break;
		case 15:
			gpio->pin = P1_15; //AN5
			break;
		default :
			return HAL_GPIO_NOK;  // invalid pin
		}
		break;
	}

	case LLHAL_GPIO_PORT_2:
	{
		switch(pin)
		{
		case 0:
			gpio->pin = P2_0;
			break;
		case 1:
			gpio->pin = P2_1;
			break;
		case 2:
			gpio->pin = P2_2;
			break;
		case 3:
			gpio->pin = P2_3;
			break;
		case 4:
			gpio->pin = P2_4;
			break;
		case 5:
			gpio->pin = P2_5;
			break;
		case 6:
			gpio->pin = P2_6;
			break;
		case 7:
			gpio->pin = P2_7;
			break;
		case 9:
			gpio->pin = P2_9;
			break;
		case 10:
			gpio->pin = P2_10;
			break;
		case 13:
			gpio->pin = P2_13;
			break;
		case 14:
			gpio->pin = P2_14;
			break;
		case 15:
			gpio->pin = P2_15;
			break;

		default :
			return HAL_GPIO_NOK; // invalid pin
		}
		break;
	}
	case LLHAL_GPIO_PORT_3:
	{
		switch(pin)
		{
		case 8:
			return gpio->pin = P3_8;
			break;
		case 9:
			return gpio->pin = P3_9;
			break;
		case 10:
			return gpio->pin = P3_10;
			break;
		case 11:
			return gpio->pin = P3_11;
			break;
		case 12:
			return gpio->pin = P3_12;
			break;
		case 13:
			return gpio->pin = P3_13;
			break;
		case 14:
			return gpio->pin = P3_14;
			break;
		case 15:
			return gpio->pin = P3_15;
			break;
		default : return HAL_GPIO_NOK; // invalid pin
		}
		break;
	}
	case LLHAL_GPIO_PORT_4:
		{
			switch(pin)
			{
			case 0:
				gpio->pin = P4_0;
			case 4:
				gpio->pin = P4_4;
				break;
			case 5:
				gpio->pin = P4_5;
				break;
			case 6:
				gpio->pin = P4_6;
				break;
			case 7:
				gpio->pin = P4_7;
				break;
			default :
				return HAL_GPIO_NOK; // invalid pin
			}
			break;
		}
	case LLHAL_GPIO_PORT_5:
	{
			switch(pin)
			{
			case 0:
				gpio->pin = P5_0;
			case 1:
				gpio->pin = P5_1;
				break;
			case 2:
				gpio->pin = P5_2;
				break;
			case 3:
				gpio->pin = P5_3;
				break;
			case 4:
				gpio->pin = P5_4;
				break;
			case 5:
				gpio->pin = P5_5;
				break;
			case 6:
				gpio->pin = P5_6;
				break;
			case 7:
				gpio->pin = P5_7;
				break;

			default :
				return HAL_GPIO_NOK; // invalid pin
			}
			break;
	}
	case LLHAL_GPIO_PORT_7:
	{
		switch(pin)
		{
		case 15:
			gpio->pin = P7_15;
			break;
		default :
			// invalid pin
			return HAL_GPIO_NOK;

		}
		break;
	}
	case LLHAL_GPIO_PORT_8:
		{
			switch(pin)
			{
			case 1:
				gpio->pin = P8_1;
				break;
			case 11:
				gpio->pin = P8_11;
				break;
			case 13:
				gpio->pin = P8_13;
				break;
			case 14:
				gpio->pin = P8_14;
				break;
			case 15:
				gpio->pin = P8_15;
				break;
			default :
				// invalid pin
				return HAL_GPIO_NOK;
			}
			break;
		}

	case LLHAL_GPIO_PORT_10:
		{
			switch(pin)
			{
			case 0:
				gpio->pin = P10_0;
				break;
			case 12:
				gpio->pin = P10_12;
				break;
			case 13:
				gpio->pin = P10_13;
				break;
			case 14:
				gpio->pin = P10_14;
				break;
			case 15:
				gpio->pin = P10_15;
				break;
			default :
				// invalid pin
				return HAL_GPIO_NOK;
			}
			break;
		}

	case LLHAL_GPIO_PORT_11:
	{
		switch(pin)
		{
		case 12:
			gpio->pin = P11_12;
			break;
		case 13:
			gpio->pin = P11_13;
			break;
		case 14:
			gpio->pin = P11_14;
			break;
		case 15:
			gpio->pin = P11_15;
			break;
		default :
			// invalid pin
			return HAL_GPIO_NOK;
		}
		break;
	}

	case LLHAL_GPIO_PORT_ARDUINO_DIGITAL:
	{
		/*
		 * ARDUINO digital port: convert port and pin using MCU port and pin
		 */
		switch(pin)
		{
		case 0:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 15, gpio);
		case 1:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 14, gpio);
		case 2:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_4, 7, gpio);
		case 3:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_4, 6, gpio);
		case 4:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_4, 5, gpio);
		case 5:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_4, 4, gpio);
		case 6:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_8, 13, gpio);
		case 7:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_8, 11, gpio);
		case 8:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_8, 15, gpio);
		case 9:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_8, 14, gpio);
		case 10:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_10, 13, gpio);
		case 11:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_10, 14, gpio);
		case 12:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_10, 15, gpio);
		case 13:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_10, 12, gpio);

		case 14:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 3, gpio);
		case 15:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 2, gpio);
		default:
			// invalid pin name
			return HAL_GPIO_NOK;
		}
	}
	case LLHAL_GPIO_PORT_ARDUINO_ANALOG:
	{
		/*
		 * ARDUINO analog port: convert port and pin using MCU port and pin
		 */
		switch(pin)
		{
		case 0:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 8, gpio);
		case 1:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 9, gpio);
		case 2:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 10, gpio);
		case 3:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 11, gpio);
		case 4:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 13, gpio);
		case 5:
			return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 15, gpio);
		default: return HAL_GPIO_NOK; // invalid pin name
		}
	}

	case LLHAL_GPIO_PORT_CN9:
	{
		switch(pin)
		{
		case 1: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 2, gpio);
		case 2: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 3, gpio);
		case 5: return hal_get_gpio_data(LLHAL_GPIO_PORT_10, 12, gpio);
		case 6: return hal_get_gpio_data(LLHAL_GPIO_PORT_10, 15, gpio);
		case 7: return hal_get_gpio_data(LLHAL_GPIO_PORT_10, 14, gpio);
		case 8: return hal_get_gpio_data(LLHAL_GPIO_PORT_10, 13, gpio);
		case 9: return hal_get_gpio_data(LLHAL_GPIO_PORT_8, 14, gpio);
		case 10: return hal_get_gpio_data(LLHAL_GPIO_PORT_8, 15, gpio);
		default:	return HAL_GPIO_NOK;	// invalid pin
		}
	}

	case LLHAL_GPIO_PORT_CN14:
	{
		switch(pin)
		{
		case 1: return hal_get_gpio_data(LLHAL_GPIO_PORT_8, 11, gpio);
		case 2: return hal_get_gpio_data(LLHAL_GPIO_PORT_8, 13, gpio);
		case 3: return hal_get_gpio_data(LLHAL_GPIO_PORT_4, 4, gpio);
		case 4: return hal_get_gpio_data(LLHAL_GPIO_PORT_4, 5, gpio);
		case 5: return hal_get_gpio_data(LLHAL_GPIO_PORT_4, 6, gpio);
		case 6: return hal_get_gpio_data(LLHAL_GPIO_PORT_4, 7, gpio);
		case 7: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 14, gpio);
		case 8: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 15, gpio);
		default:	return HAL_GPIO_NOK;	// invalid pin
		}
	}
	case LLHAL_GPIO_PORT_CN15:
	{
		switch (pin)
		{
		case 1: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 8, gpio);
		case 2: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 9, gpio);
		case 3: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 10, gpio);
		case 4: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 11, gpio);
		case 5: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 13, gpio);
		case 6: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 15, gpio);
		default:	return HAL_GPIO_NOK;	// invalid pin
		}
	}

	case LLHAL_GPIO_PORT_CN8:
	{
		switch (pin)
		{
		case 1: return hal_get_gpio_data(LLHAL_GPIO_PORT_4, 0, gpio);
		case 2: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 13, gpio);
		case 3: return hal_get_gpio_data(LLHAL_GPIO_PORT_5, 7, gpio);
		case 4: return hal_get_gpio_data(LLHAL_GPIO_PORT_5, 6, gpio);
		case 5: return hal_get_gpio_data(LLHAL_GPIO_PORT_5, 5, gpio);
		case 6: return hal_get_gpio_data(LLHAL_GPIO_PORT_5, 4, gpio);
		case 7: return hal_get_gpio_data(LLHAL_GPIO_PORT_5, 3, gpio);
		case 8: return hal_get_gpio_data(LLHAL_GPIO_PORT_5, 2, gpio);
		case 9: return hal_get_gpio_data(LLHAL_GPIO_PORT_5, 1, gpio);
		case 10: return hal_get_gpio_data(LLHAL_GPIO_PORT_5, 0, gpio);
		default:	return HAL_GPIO_NOK;	// invalid pin
		}
	}
	case LLHAL_GPIO_PORT_CN11:
	{
			switch(pin)
			{
			case 1: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 7, gpio);
			case 2: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 6, gpio);
			case 3: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 5, gpio);
			case 4: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 4, gpio);
			case 5: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 3, gpio);
			case 6: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 2, gpio);
			case 7: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 1, gpio);
			case 8: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 0, gpio);
			default:	return HAL_GPIO_NOK;	// invalid pin
			}
	}
	case LLHAL_GPIO_PORT_CN12:
	{
			switch(pin)
			{
			case 1: return hal_get_gpio_data(LLHAL_GPIO_PORT_3, 15, gpio);
			case 2: return hal_get_gpio_data(LLHAL_GPIO_PORT_3, 14, gpio);
			case 3: return hal_get_gpio_data(LLHAL_GPIO_PORT_3, 13, gpio);
			case 4: return hal_get_gpio_data(LLHAL_GPIO_PORT_3, 12, gpio);
			case 5: return hal_get_gpio_data(LLHAL_GPIO_PORT_3, 11, gpio);
			case 6: return hal_get_gpio_data(LLHAL_GPIO_PORT_3, 10, gpio);
			case 7: return hal_get_gpio_data(LLHAL_GPIO_PORT_3, 9, gpio);
			case 8: return hal_get_gpio_data(LLHAL_GPIO_PORT_3, 8, gpio);
			default:	return HAL_GPIO_NOK;	// invalid pin
			}
	}
	case LLHAL_GPIO_PORT_CN13:
	{
				switch(pin)
				{
				case 1: return hal_get_gpio_data(LLHAL_GPIO_PORT_7, 15, gpio);
				case 2: return hal_get_gpio_data(LLHAL_GPIO_PORT_8, 1, gpio);
				case 3: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 9, gpio);
				case 4: return hal_get_gpio_data(LLHAL_GPIO_PORT_2, 10, gpio);
				// pin 5, 6 NTSC1B; NTSC1A
				//case 5:
				//case 6:
				default:	return HAL_GPIO_NOK;	// invalid pin
				}
	}
	case LLHAL_GPIO_PORT_CN16:
		{
			switch (pin)
			{
			case 1: return hal_get_gpio_data(LLHAL_GPIO_PORT_10, 0, gpio);
			case 2: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 1, gpio);
			case 3: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 0, gpio);
			case 4: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 12, gpio);
			case 5: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 7, gpio);
			case 6: return hal_get_gpio_data(LLHAL_GPIO_PORT_1, 8, gpio);
			default:	return HAL_GPIO_NOK;	// invalid pin
			}
		}
	default:
		return HAL_GPIO_NOK;
	}
	return HAL_GPIO_OK;
}








