/* Linker script for mbed RZ_A1H */

__KERNEL_WORKING_BUFFER_Size__ 		= 0x300000;
__SSL_MEM_Size__ 		            = 0x80000;
/* Dispatch timeout memory size is a memory area for a table of struct threadFdTimeout_t for every java thread */
/* The size of the struct should be adjust if changed currenty 88 ( 4 + (4 align)+ 8 * 10(MEMP_NUM_TCP_PCB) ) */
__SOCKET_TIMEOUT_MEM_Size__    = (0x58 * _java_max_nb_standaloneEngines);

/* Linker script to configure memory regions. */
MEMORY
{
  ROM   (rx)  : ORIGIN = 0x00000000, LENGTH = 0x02000000
  BOOT_LOADER (rx) : ORIGIN = 0x18000000, LENGTH = 0x00004000 
  SFLASH (rx) : ORIGIN = 0x18004000, LENGTH = 0x07FFB000 
  L_TTB (rw)  : ORIGIN = 0x20000000, LENGTH = 0x00004000 
  RAM (rwx) : ORIGIN = 0x20020000, LENGTH = 0x00700000
  RAM_NC (rwx) : ORIGIN = 0x20900000, LENGTH = 0x00100000
  MAC_ADDR_SECTOR (rx) : ORIGIN = 0x187FFF000, LENGTH = 0x00001000
}

/* Linker script to place sections and symbol values. Should be used together
 * with other linker script that defines memory regions FLASH and RAM.
 * It references following symbols, which must be defined in code:
 *   Reset_Handler : Entry of reset handler
 * 
 * It defines following symbols, which code can use without definition:
 *   __exidx_start
 *   __exidx_end
 *   __etext
 *   __data_start__
 *   __preinit_array_start
 *   __preinit_array_end
 *   __init_array_start
 *   __init_array_end
 *   __fini_array_start
 *   __fini_array_end
 *   __data_end__
 *   __bss_start__
 *   __bss_end__
 *   __end__
 *   end
 *   __HeapLimit
 *   __StackLimit
 *   __StackTop
 *   __stack
 */
ENTRY(Reset_Handler)

SECTIONS
{
    .boot :
    {
        KEEP(*(.boot_loader)) 
    } > BOOT_LOADER 
    
    .text :
    {
        Image$$ROX_DATA$$Base = .;
        * (RESET)
        . += 0x00000400;

        KEEP(*(.isr_vector))
        *(SVC_TABLE)
        *(EXCLUDE_FILE(*spibsc_flash*.o *spibsc_ioset*.o *writeUIDFlash.o *rza_io_regrw.o) .text*)

        KEEP(*(.init))
        KEEP(*(.fini))

        /* .ctors */
        *crtbegin.o(.ctors)
        *crtbegin?.o(.ctors)
        *(EXCLUDE_FILE(*crtend?.o *crtend.o) .ctors)
        *(SORT(.ctors.*))
        *(.ctors)

        /* .dtors */
        *crtbegin.o(.dtors)
        *crtbegin?.o(.dtors)
        *(EXCLUDE_FILE(*crtend?.o *crtend.o) .dtors)
        *(SORT(.dtors.*))
        *(.dtors)
        
        /* Uncomment next line if MMU is activated vith DEBUG_SEPARATE_CONF_AREA */
        /* . = ALIGN(0x100000);*/
        Image$$ROX_DATA$$Limit = .;

        Image$$RO_DATA$$Base = .;
        *(.rodata*)
        /* Uncomment next line if MMU is activated vith DEBUG_SEPARATE_CONF_AREA */
        /* . = ALIGN(0x100000); */
        Image$$RO_DATA$$Limit = .;

        KEEP(*(.eh_frame*))
    } > SFLASH

    .ARM.extab : 
    {
        *(.ARM.extab* .gnu.linkonce.armextab.*)
    } > SFLASH

    __exidx_start = .;
    .ARM.exidx :
    {
        *(.ARM.exidx* .gnu.linkonce.armexidx.*)
    } > SFLASH
    __exidx_end = .;
    
    .copy.table :
    {
        . = ALIGN(4);
        __copy_table_start__ = .;
        LONG (__etext)
        LONG (__data_start__)
        LONG (__data_end__ - __data_start__)
        LONG (__etext2)
        LONG (__nc_data_start)
        LONG (__nc_data_end - __nc_data_start)
        __copy_table_end__ = .;
    } > SFLASH

    .zero.table :
    {
        . = ALIGN(4);
        __zero_table_start__ = .;
        LONG (__bss_start__)
        LONG (__bss_end__ - __bss_start__)
        LONG (__nc_bss_start)
        LONG (__nc_bss_end - __nc_bss_start)
        __zero_table_end__ = .;
    } > SFLASH
    
    __etext = .;
        
    .ttb :
    {
        Image$$TTB$$ZI$$Base = .;
        . += 0x00004000;
        Image$$TTB$$ZI$$Limit = .;
    } > L_TTB

    .data : AT (__etext)
    {
    	Image$$RW_DATA$$Base = .;
        __data_start__ = .;
        *(vtable)
        *(.data*)

        . = ALIGN(4);
        /* preinit data */
        PROVIDE (__preinit_array_start = .);
        KEEP(*(.preinit_array))
        PROVIDE (__preinit_array_end = .);

        . = ALIGN(4);
        /* init data */
        PROVIDE (__init_array_start = .);
        KEEP(*(SORT(.init_array.*)))
        KEEP(*(.init_array))
        PROVIDE (__init_array_end = .);


        . = ALIGN(4);
        /* finit data */
        PROVIDE (__fini_array_start = .);
        KEEP(*(SORT(.fini_array.*)))
        KEEP(*(.fini_array))
        PROVIDE (__fini_array_end = .);

        . = ALIGN(4);
        /* All data end */
        __data_end__ = .;

    } > RAM
    
    __start_flash_progFlash__ = __etext + SIZEOF(.data);

    .sflash_prog ALIGN(0x100) : AT (__start_flash_progFlash__)
    {
       	__start_ram_progFlash__ = .;
       	*(.text.write_uid_to_flash)
    	*(.text.R_SFLASH_*)
    	*(.text.Userdef_SFLASH_*)
    	*(.text.Userdef_SPIBSC_*)
    	*(.text.spibsc_*)
    	*(.text.RZA_IO_Reg*)
    	*(.text.set_mode)
		*(.text.read_status)
		*(.text.read_config)
		*(.text.read_bank)
		*(.text.write_only_status)
		*(.text.write_status)
		*(.text.clear_status)
		*(.text.enter_4b)
		*(.text.read_data_quad)
		*(.text.read_data_single)
		*(.text.SPIBSC_Exread_Mode_Config)
		*(.text.Pekoe_io_spibsc_port_setting)
		
		. = ALIGN(0x100);
        __end_ram_progFlash__ = .;
    } > RAM
    
    .bss ALIGN(0x400):
    {
        __bss_start__ = .;
        *(.bss*)
        *(COMMON)
        __bss_end__ = .;
    } > RAM
    
    .KERNEL_WORKING_BUFFER :
	{
		. = ALIGN(4);
	  	PROVIDE ( __KERNEL_WORKING_BUFFER_Start__ = . );
	    . = . + __KERNEL_WORKING_BUFFER_Size__;
	    . = ALIGN(4);
	    __KERNEL_WORKING_BUFFER_End__ = .;         /* define a global symbol at KF memory end */
	} > RAM
    
    .SSL_MEM :
    {
        . = ALIGN(4);
        PROVIDE ( __SSL_MEM_Start__ = . );
        . = . + __SSL_MEM_Size__;
        . = ALIGN(4);
        __SSL_MEM_End__ = .;         /* define a global symbol at SSL memory end */
    } > RAM

    ._socket_asyncOp_stack :
    {
        . = ALIGN(4);
        PROVIDE ( __SOCKET_TIMEOUT_MEM_Start__ = . );
        . = . + __SOCKET_TIMEOUT_MEM_Size__;
        . = ALIGN(4);
        __SOCKET_TIMEOUT_MEM_End__ = .;         /* define a global symbol at SSL memory end */
        
        
        /* Uncomment next line if MMU is activated vith DEBUG_SEPARATE_CONF_AREA */
        /* . = ALIGN(0x100000); */
        Image$$RW_DATA$$Limit = .;
    } > RAM
    
    .heap :
    {
        __end__ = .;
        end = __end__;
        *(.heap*)
        __HeapLimit = .;
    } > RAM

    /* .stack_dummy section doesn't contains any symbols. It is only
     * used for linker to calculate size of stack sections, and assign
     * values to stack symbols later */
    .stack_dummy :
    {
        *(.stack)
    } > RAM

    __etext2 = __start_flash_progFlash__ + SIZEOF(.sflash_prog);
    .nc_data : AT (__etext2)
    {
        Image$$RW_DATA_NC$$Base = .;
        __nc_data_start = .;
        *(NC_DATA)

        . = ALIGN(4);
        __nc_data_end = .;
        Image$$RW_DATA_NC$$Limit = .;
    } > RAM_NC

    .nc_bss (NOLOAD) :
    {
        Image$$ZI_DATA_NC$$Base = .;
        __nc_bss_start = .;
        *(NC_BSS)

        . = ALIGN(4);
        __nc_bss_end = .;
        Image$$ZI_DATA_NC$$Limit = .;
    } > RAM_NC

    /* Set stack top to end of RAM, and stack limit move down by
     * size of stack_dummy section */
    __StackTop = ORIGIN(RAM) + LENGTH(RAM);
    __StackLimit = __StackTop - SIZEOF(.stack_dummy);
    PROVIDE(__stack = __StackTop);
    
    /* Check if data + heap + stack exceeds RAM limit */
    ASSERT(__StackLimit >= __HeapLimit, "region RAM overflowed with stack")


}
