/*
 * C
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#include "stdint.h"
#include "stdio.h"
#include "LLNET_Cfg.h"
#include "LLNET_Util.h"
#include <string.h>
#include "mbed_mac_address.h"
#include "device_uid.h"

#ifdef __cplusplus
	extern "C" {
#endif

/*
 * Default MAC address buffer, the Organization Unique Identifier (OUI) is kept to
 * the one used by default with mbed *
 */
static uint8_t MacAddrFlash_buffer[6] = { 0x00, 0x02, 0xF7, 0xFF, 0xFF, 0xFF};

/*  API -------------------------------------------------------------------- */

void mbed_mac_address(char *mac)
{
	/* Get MAC address stored in flash memory */
	uint8_t * flashUIDAddr = (uint8_t *)(GRPEACH_FLASH_UID_ADDR_START);

    // check MAC address configuration requested by MicroEJ workbench extension
	int32_t mac_conf_enabled = ((int32_t)&LLNET_MAC_CONF == LLNET_TRUE);
	if(mac_conf_enabled){
		uint8_t mac_address[6] = {0};
		// MAC address configuration overriden by workbench extension
		parseMacAddr(LLNET_getMACAddr(), &mac_address[0], 0, 6);

		if(mac_address[3] == 0 && mac_address[4] == 0 && mac_address[5] == 0){
			// Consider only the prefix and use ID from MAC address stored in flash
				mac_address[3] = flashUIDAddr[0];
				mac_address[4] = flashUIDAddr[1];
				mac_address[5] = flashUIDAddr[2];
		}

		memcpy(mac, &(mac_address[0]), 6);
	}else{
		// set default OUI flash MAC address
		MacAddrFlash_buffer[3] = flashUIDAddr[0];
		MacAddrFlash_buffer[4] = flashUIDAddr[1];
		MacAddrFlash_buffer[5] = flashUIDAddr[2];
		memcpy(mac, MacAddrFlash_buffer, 6);
	}
}

#ifdef __cplusplus
	}
#endif
