/*
 * C
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#include <LLNET_SSL_CONSTANTS.h>
#include <time.h>
#include <stdio.h>

#ifdef __cplusplus
 extern "C" {
#endif

#ifdef LLNET_SSL_DEBUG
/* ---- Define the mbedtls debug level ---- */
#define MBEDTLS_DEBUG_LEVEL 2
 /* ---- Define the mbedtls debug printer ---- */
#define LLNET_SSL_DEBUG_MBEDTLS_TRACE 	   print_mbedtls_error
#else
#define LLNET_SSL_DEBUG_MBEDTLS_TRACE(...) ((void) 0)
#endif


 /* ---- Specific C Time function ---- */
 /*
  * time function to get current JVM application time.
  * @param time : pointer to time_t struct that will be updated with the current time in milliseconds
  * @return the current time in milliseconds
  */
time_t LLNET_SSL_utils_mbedtls_time(time_t* time);

/* ---- Specific memory allocation function ---- */

/*
 * Specific memory allocator to allocate mbedtls objects in dedicated SSL memory pool
 * @param number of block to allocate
 * @param size of block to allocate
 * @return pointer to memory block zeroized
 */
void* LLNET_SSL_utils_mbedtls_calloc(size_t num, size_t size);

/*
 * Free memory of mbedtls objects in dedicated SSL memory pool
 * @param pointer of the block to free
 * @return None
 */
void LLNET_SSL_utils_mbedtls_free(void* ptr);

/* ---- Specific net layer connection functions ---- */

/*
 * Net receive layer adaptation function for mbedtls .
 * @param ctx : Net context (pointer to the socket fd)
 * @param buf : buffer for receiving data
 * @param len : length to receive
 * @return -1 on error, number of received bytes otherwise
 */
int LLNET_SSL_utils_mbedtls_recv(void *ctx, unsigned char *buf, size_t len);

/*
 * Net send layer adaptation function for mbedtls .
 * @param ctx : Net context (pointer to the socket fd)
 * @param buf : buffer of data to send
 * @param len : length to send
 * @return -1 on error, number of sent bytes otherwise
 */
int LLNET_SSL_utils_mbedtls_send(void *ctx, const unsigned char *buf, size_t len);

/* ---- mbedtls custom function for error printing ---- */
void print_mbedtls_error(const char *name, int err);

#ifdef __cplusplus
}
#endif
