/*
 * C
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
#include <stddef.h>
#include <stdio.h>
#include <string.h>

#include <LLNET_SSL_MEM.h>
#include <LLNET_SSL_utils_mbedtls.h>

#include "native_time.h"
#include "microej.h"

#include "mbedtls/error.h"
#include "mbedtls/net_sockets.h"

#include <posix/sys/socket.h>

#ifdef __cplusplus
 extern "C" {
#endif

/* ---- Specific C Time function ---- */

time_t LLNET_SSL_utils_mbedtls_time(time_t* time)
{
	time_t currentTime = (time_t)(LLMJVM_native_getTime(MICROEJ_FALSE)/1000);

	if (time != NULL)
	{
		(*time)=currentTime;
	}

	return currentTime;
}

/* ---- Specific memory allocation function ---- */

void* LLNET_SSL_utils_mbedtls_calloc(size_t num, size_t size)
{
	void * pointer = LLNET_SSL_MEM_allocate((int32_t)(num * size));
	if (pointer != NULL)
		memset(pointer, 0, (int32_t)(num * size));
	return pointer;
}

void LLNET_SSL_utils_mbedtls_free(void* ptr)
{
	LLNET_SSL_MEM_free(ptr);
}

/* ---- Specific net layer connection functions ---- */

/**
 * Receive callback for mbed TLS
 */
int LLNET_SSL_utils_mbedtls_recv(void *ctx, unsigned char *buf, size_t len) {
    int recv_bytes = -1;

    int fd = *((int*)ctx);

    if( fd < 0 )
        return( MBEDTLS_ERR_NET_INVALID_CONTEXT );

    recv_bytes = recv(fd, buf, len, 0);
    int netError= errno;
    
    if( recv_bytes < 0 )
    {
#if defined EAGAIN
    	if( ( ( fcntl( fd, F_GETFL, 0) & O_NONBLOCK ) == O_NONBLOCK ) && (netError == EAGAIN))
#else if defined EWOULDBLOCK && EWOULDBLOCK != EAGAIN
    	if( ( ( fcntl( fd, F_GETFL ) & O_NONBLOCK ) == O_NONBLOCK ) && (netError == EWOULDBLOCK))
#endif
    	{
            return( MBEDTLS_ERR_SSL_WANT_READ );
    	}

        if( netError == EPIPE || netError == ECONNRESET )
            return( MBEDTLS_ERR_NET_CONN_RESET );

        if( netError == EINTR )
            return( MBEDTLS_ERR_SSL_WANT_READ );

        return( MBEDTLS_ERR_NET_RECV_FAILED );
    }

    return recv_bytes;
}

/**
 * Send callback for mbed TLS
 */
int LLNET_SSL_utils_mbedtls_send(void *ctx, const unsigned char *buf, size_t len) {
    int size_sent = -1;

    int fd = *((int*)ctx);
    if( fd < 0 )
        return( MBEDTLS_ERR_NET_INVALID_CONTEXT );

    size_sent = send(fd, buf, len, 0);
    int netError= errno;
    
    if( size_sent < 0 )
    {
#if defined EAGAIN
    	if( ( ( fcntl( fd, F_GETFL, 0) & O_NONBLOCK ) == O_NONBLOCK ) && (netError == EAGAIN))
#else if defined EWOULDBLOCK && EWOULDBLOCK != EAGAIN
    	if( ( ( fcntl( fd, F_GETFL ) & O_NONBLOCK ) == O_NONBLOCK ) && (errno == EWOULDBLOCK))
#endif
    	{
            return( MBEDTLS_ERR_SSL_WANT_WRITE );
    	}

        if( netError == EPIPE || netError == ECONNRESET )
            return( MBEDTLS_ERR_NET_CONN_RESET );

        if( netError == EINTR )
            return( MBEDTLS_ERR_SSL_WANT_WRITE );

        return( MBEDTLS_ERR_NET_SEND_FAILED );
    }

	return size_sent;
}

/* ---- mbedtls custom function for error printing ---- */

/**
 * Helper for pretty-printing mbed TLS error codes
 */
 void print_mbedtls_error(const char *name, int err) {
    char buf[128];
    mbedtls_strerror(err, buf, sizeof (buf));
    printf("%s() failed: -0x%04x (%d): %s\r\n", name, -err, err, buf);
}

#ifdef __cplusplus
}
#endif
