/*
 * C
 *
 * Copyright 2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/* Includes -------------------------------------------------------------------*/
#ifdef __cplusplus
	extern "C" {
#endif

#include "LLLEDS_impl.h"

#ifdef __cplusplus
	}
#endif

#include "mbed.h"
#include "SoftPWM.h"

/* Defines --------------------------------------------------------------------*/

/* Define for use soft PWM for leds */
#define USE_SOFT_PWM

/* Globals --------------------------------------------------------------------*/

static DigitalOut led_user(LED_USER);
#ifdef USE_SOFT_PWM
	static SoftPWM led_red(LED_RED);
	static SoftPWM led_green(LED_GREEN);
	static SoftPWM led_blue(LED_BLUE);
#else
	static DigitalOut led_red(LED_RED);
	static DigitalOut led_green(LED_GREEN);
	static DigitalOut led_blue(LED_BLUE);
#endif


/* API ------------------------------------------------------------------------*/

int32_t LLLEDS_IMPL_initialize(void)
{
	led_user = 0;
	led_red = 0;
	led_green = 0;
	led_blue = 0;
#ifdef USE_SOFT_PWM
	led_red.period_ms(10);
	led_green.period_ms(10);
	led_blue.period_ms(10);
#endif
	return 4;
}

int32_t LLLEDS_IMPL_getIntensity(int32_t ledID)
{
	switch(ledID)
	{
		default:
		case 0: return (led_user== LLLEDS_MIN_INTENSITY)?LLLEDS_MIN_INTENSITY:LLLEDS_MAX_INTENSITY;
		case 1:
#ifdef USE_SOFT_PWM
			return (led_red.read()*LLLEDS_MAX_INTENSITY);
#else
			return (led_red== LLLEDS_MIN_INTENSITY)?LLLEDS_MIN_INTENSITY:LLLEDS_MAX_INTENSITY;
#endif
		case 2:
#ifdef USE_SOFT_PWM
			return (led_green.read()*LLLEDS_MAX_INTENSITY);
#else
			return (led_green== LLLEDS_MIN_INTENSITY)?LLLEDS_MIN_INTENSITY:LLLEDS_MAX_INTENSITY;
#endif
		case 3:
#ifdef USE_SOFT_PWM
			return (led_blue.read()*LLLEDS_MAX_INTENSITY);
#else
			return (led_blue== LLLEDS_MIN_INTENSITY)?LLLEDS_MIN_INTENSITY:LLLEDS_MAX_INTENSITY;
#endif
	}
}

void LLLEDS_IMPL_setIntensity(int32_t ledID, int32_t intensity)
{
	switch(ledID)
	{
		case 0: led_user = (intensity == LLLEDS_MIN_INTENSITY)?0:1;
			break;
		case 1:
#ifdef USE_SOFT_PWM
			led_red = ((float)intensity * 1.0) / LLLEDS_MAX_INTENSITY;
#else
			led_red = (intensity == LLLEDS_MIN_INTENSITY)?0:1;
#endif
			break;
		case 2:
#ifdef USE_SOFT_PWM
			led_green = ((float)intensity * 1.0) / LLLEDS_MAX_INTENSITY;
#else
			led_green = (intensity == LLLEDS_MIN_INTENSITY)?0:1;
#endif
			break;
		case 3:
#ifdef USE_SOFT_PWM
			led_blue = ((float)intensity * 1.0) / LLLEDS_MAX_INTENSITY;
#else
			led_blue = (intensity == LLLEDS_MIN_INTENSITY)?0:1;
#endif
			break;
		default:
			break;
	}
}
