/*
 * C
 *
 * Copyright 2013-2017 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "microej.h"
#include "LLMJVM.h"
#include "buttons_helper.h"
#include "buttons_manager.h"
#include "buttons_helper_configuration.h"

#include "mbed.h"
#include "native_time.h"

/* Defines --------------------------------------------------------------------*/
#define LLDUMP_PUSH_MIN_TIME_MS      (1000)
#define LLDUMP_PUSH_NB_THRESHOLD     (5)

/* Global --------------------------------------------------------------------*/

static InterruptIn BUTTON_STRUCT[NUMBER_OF_BUTTONS] = {USER_BUTTON0};
static const uint8_t BUTTON_REVERSE[NUMBER_OF_BUTTONS] = {MICROEJ_TRUE};
static volatile uint8_t nb_push_button = 0;
static volatile int64_t last_push_button_time = 0;

/* Private API ---------------------------------------------------------------*/

static void BUTTONS_MANAGER_event(uint8_t buttonID)
{
	uint8_t button_pressed;

	int state = BUTTON_STRUCT[buttonID].read();

	if (state == 1)
	{
		// GPIO == 1

		if (BUTTON_REVERSE[buttonID] == MICROEJ_TRUE)
		{
			// GPIO == 1 means "released"
			button_pressed = MICROEJ_FALSE;
		}
		else
		{
			// GPIO == 1 means "pressed"
			button_pressed = MICROEJ_TRUE;
		}
	}
	else
	{
		// GPIO == 0

		if (BUTTON_REVERSE[buttonID] == MICROEJ_TRUE)
		{
			// GPIO == 0 means "pressed"
			button_pressed = MICROEJ_TRUE;
		}
		else
		{
			// GPIO == 0 means "released"
			button_pressed = MICROEJ_FALSE;
		}
	}

	if (button_pressed == MICROEJ_TRUE)
	{
		BUTTONS_HELPER_pressed(buttonID);

		if (LLMJVM_native_getTime(MICROEJ_TRUE) > (int64_t)(last_push_button_time+LLDUMP_PUSH_MIN_TIME_MS)){
			last_push_button_time = LLMJVM_native_getTime(MICROEJ_TRUE);
			nb_push_button = 0;
		}
		else{
			if (nb_push_button >= (LLDUMP_PUSH_NB_THRESHOLD-1))
			{
				LLMJVM_dump();
				nb_push_button = 0;
			}
			else
				nb_push_button++;
		}
	}
	else
	{
		BUTTONS_HELPER_released(buttonID);
	}
}


void button_s1_callback(void)
{
	BUTTONS_MANAGER_event(BUTTON_S1_ID);
}

static void BUTTONS_MANAGER_init()
{
    // Enable buttons
	BUTTON_STRUCT[0].rise(&button_s1_callback);
	BUTTON_STRUCT[0].fall(&button_s1_callback);
}

/* API -----------------------------------------------------------------------*/

void BUTTONS_MANAGER_interrupt(uint8_t buttonID)
{
	// no interrupt
}

void BUTTONS_MANAGER_enable_interrupts(void)
{
    // Enable buttons
	uint8_t button = 0;
	for (button =0; button < NUMBER_OF_BUTTONS; button++)
	{
		BUTTON_STRUCT[button].enable_irq();
	}
}

void BUTTONS_MANAGER_disable_interrupts(void)
{
    // Enable buttons
	uint8_t button = 0;
	for (button =0; button < NUMBER_OF_BUTTONS; button++)
	{
		BUTTON_STRUCT[button].disable_irq();
	}
}

void BUTTONS_MANAGER_initialize(void)
{
	BUTTONS_HELPER_initialize();
	BUTTONS_MANAGER_init();
}
